
/*
WebSerial example
Reads from a webSerial serial port, and writes to it.
Works on Chrome and Edge and Opera browsers. 

created 28 Jan 2022
modified 15 May 2022
by Tom Igoe
modified by Charlotte Deroubaix 7 May 2024
*/


// the DOM elements that might be changed by various functions:
let portButton;   // the open/close port button
let readingsSpan; // DOM element where the incoming readings go
let timeSpan;     // DOM element for one special reading
let webserial;
let myChart;
let previousData = null; // Initialize previousData variable to store the previous value


function setup() {
  // get the DOM elements and assign any listeners needed:
  // user text input:
  const textInput = document.getElementById("txt");
  //textInput.addEventListener("keyup", readTextInput);
  // user range input:
  const slider = document.getElementById("dim");
  //slider.addEventListener("change", readRangeInput);
  // span for incoming serial messages:
  readingsSpan = document.getElementById("readings");
  // span for incoming serial messages:
  //timeSpan = document.getElementById("seconds");
  
  webserial = new WebSerialPort();
  if (webserial) {
    webserial.on("data", serialRead);
     // port open/close button:
     portButton = document.getElementById("portButton");
     portButton.addEventListener("click", openClosePort);
   }
}

//add by Charlotte Deroubaix
function setupChart() {
  const ctx = document.getElementById('myChart').getContext('2d');
  myChart = new Chart(ctx, {
    type: 'line',
    data: {
      labels: [], // Add received data labels here
      datasets: [{
        label: 'Pressure',
        data: [], // Add received data values here
        borderColor: 'rgb(75, 192, 192)',
        tension: 0.1
      }]
    },
    options: {
      scales: {
        x: {
          type: 'time', // Use 'time' for time scale
          time: {
            unit: 'second' // Adjust the time scale unit as needed
          }
        },
        y: {
          min:0,
          max:1
        }
      },
      responsive: false,
      maintainAspectRatio: false
    }
  });
}

function updateChart(data) {
  if (data !== 0) {
    // Update chart data with the received data
    myChart.data.labels.push(new Date());
    myChart.data.datasets[0].data.push(data);
    myChart.update();
  }
}

function serialRead(event) {
  // Parse the received data as a float
  const data = parseFloat(event.detail.data);
  
  // Check if the parsed data is a valid number
  if (!isNaN(data)) {
    // If valid, update the readingsSpan with the received data
    readingsSpan.innerHTML = data;
    // Update lastValidData to store the current value
    previousData = data;
    // Update the chart with the received data
    updateChart(data);
    
  } else {
    // If NaN, display a message indicating invalid data
    readingsSpan.innerHTML = "Invalid data";
  
  }

  console.log("Received data:", data);

  // If lastValidData exists, update the readingsSpan with it
  if (previousData !== null) {
    readingsSpan.innerHTML = previousData;
  } 
}


async function openClosePort() {
  // label for the button will change depending on what you do:
  let buttonLabel = "Open port";
  // if port is open, close it; if closed, open it:
  if (webserial.port) {
    await webserial.closePort();
  } else {
    await webserial.openPort();
    buttonLabel = "Close port";
  }
  // change button label:
  portButton.innerHTML = buttonLabel;
}


function readTextInput(event) {
  // this function is triggered with every keystroke in the input field.
  // listen for the enter key (keyCode = 13) and skip the rest of
  // the function if you get any other key:
  if (event.keyCode != 13) {
    return;
  }
  // if you do get an enter keyCode, send the value of the field
  // out the serial port:
  webserial.sendSerial(event.target.value);
}

function readRangeInput(event) {
  // send the range input's value out the serial port:
  webserial.sendSerial(event.target.value);
}

// run the setup function when all the page is loaded:
document.addEventListener("DOMContentLoaded", function() {
  setup();
  setupChart();
});



